@@ -2,13 +2,14 @@ package Path::Class::URI;
 
 use strict;
 use 5.008_001;
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use URI;
 use URI::file;
 use Exporter::Lite;
 use Path::Class;
 use Scalar::Util qw(blessed);
+use URI::Escape;
 
 our @EXPORT = qw( file_from_uri dir_from_uri );
 
@@ -22,8 +23,11 @@ sub dir_from_uri {
 
 sub Path::Class::Entity::uri {
     my $self = shift;
-    my $path = $self->stringify;
+    my $escaped_self = $self->new( map { uri_escape($_) } $self->components );
+    # escape the components so that URI can see them as path segments
+    my $path = $escaped_self->stringify;
     $path =~ tr!\\!/! if $^O eq "MSWin32";
+    $path .= '/' if $self->isa('Path::Class::Dir'); # preserve directory if used as base URI
     if ($self->is_absolute) {
         return URI->new("file://$path");
     } else {
